CoNet <- function(data,dim,copula){
    
  # Load packages
  
  library(copula)
  library(gtools)
  library(partitions)
  
  ############################################################################
  ############################ Clean function ################################
  ############################################################################
  clean <- function(number){
    partitions <- parts(number);
    partitions <- as.matrix(partitions);
    m <- dim(partitions)[2];
    cursor <- 0
    for (j in 1:m){
      if (length(which(partitions[,j]==1))!= 0){
        cursor <- c(cursor,j)
      }
    }
    cursor <- cursor[-1]
    for (i in 1:length(cursor)){
      partitions <- partitions[,-cursor[i]]
      cursor <- cursor-1
    }
    
    l <- list();
    m <- dim(partitions)[2];
    for (j in 1:m){
      l[[j]] <- partitions[which(partitions[,j]!=0),j]
    }
    return(l)
  }
  ############################################################################
  ############################################################################
  
  ############################################################################
  ########################### flip function ##################################
  ############################################################################
  flip <- function(x){
    s1 <- dim(x)[1]
    s2 <- dim(x)[2]
    
    m <- rev(x[,1])
    for (i in 2:s2){
      m <- cbind(m,rev(x[,i]))
    }
    m <- matrix(m,s1,s2)
    return(m)
  }
  ############################################################################
  ############################################################################
  
  ############################################################################
  ################################ Replica function ##########################
  ############################################################################
  replica <- function(mat,r,l){
    # Size of the input matrix
    n <- dim(mat)[1]
    m <- dim(mat)[2]
        
    # New matrix with row replications
    result <- matrix(integer(0),0,m)
    
    for (i in 1:n){
      cur <- mat[i,]
      new <- matrix( rep( t(mat[i,]) ,r ), ncol = m, byrow = TRUE)
      result <- rbind(result,new)
    }
    
    # Replication of the new matrix
    output <- matrix( rep( t( result ) , l ) , ncol = m , byrow = TRUE )
    return(output)
  }
  ############################################################################
  ############################################################################
  
  ############################################################################
  ####################### combValue function #################################
  ############################################################################
  combValue <- function(input,r){
    n <- length(input)
    comb <- combinations(n,r)
    
    s1 <- dim(comb)[1]
    s2 <- dim(comb)[2]
    
    combValue <- matrix(0,s1,s2)
    for (i in 1:s1){
      for (j in 1:s2){
        combValue[i,j] <- input[comb[i,j]]
      }
    }
    return(combValue)
  }
  ############################################################################
  ############################################################################
  
  ############################################################################
  ####################### Remove function ####################################
  ############################################################################
  remove <- function(x,y){
    n <- length(y)
    for (i in 1:n){
      x <- x[which(x!=y[i])]
    }
    return(x) 
  }
  ############################################################################
  ############################################################################
  
  ############################################################################
  ############################# Exclusion3 ###################################
  ############################################################################
  exclusion3 <- function(excluded,input,distance){
    l <- length(excluded)
    repetition <- t(matrix(rep(excluded,distance),l,distance))
    global <- cbind(repetition,input)
    return(global)
  }
  ############################################################################
  ############################################################################
  
  ############################################################################
  ############################### Summorial ##################################
  ############################################################################
  summorial <- function(x){
    length <- 0
    i <- 1
    m <- x[1]
    while (x[i]==m){
      length <- length + 1
      i <- i + 1
    }
    return(length)
  }
  ############################################################################
  ############################################################################
  
  ############################################################################
  ################################ Routine Copula ############################
  ############################################################################
  Routine_Copula <- function(data,dim,copula){
    n <- dim(data)[1];  # number of rows of the input matrix
    m <- dim(data)[2];  # number of columns of the input matrix
    
    # Probability integral transform of the data
    ecdf <- matrix(0,n,m);
    for (i in 1:n){
      e <- matrix(data[i,],m,1);
      ecdf[i,] <- pobs(e);
    }
    
    sets <- list()
    
    # loop for varying copula dimension (dim)
    for (i in 2:dim){
      
      # Set the copula function
      if (copula == "frank"){
        f <- frankCopula(2,i);
      }
      else if (copula == "gumbel"){
        f <- gumbelCopula(2,i)
      }
      else if (copula == "clayton"){
        f <- claytonCopula(2,i)
      }
      
      # Define the vector with all the possible combinations of 
      # copula of dimension i.
      comb <- combinations(dim,i);
      r_comb <- dim(comb)[1];
      
      # llikh contains the likelihood and the parameter's values for 
      # each combination
      param <- matrix(0,r_comb,1);
      loglik <- matrix(0,r_comb,1);
      
      # For the i-th dimension copula estimate the parameter for each
      # combination and save the values for the parameter and the
      # loglikelihood
      for (j in 1:r_comb){
        input <- t(ecdf[comb[j,],])
        try(summary <- fitCopula(f,input,method='mpl',start=2),silent=TRUE);
        param[j] <- summary@estimate;
        loglik[j] <- summary@loglik;
      }
      
      result <- cbind(comb,param,loglik)
      sets[[i]] <- result
    }
    return(sets)
  }
  #############################################################################
  #############################################################################
  
  #############################################################################
  ###################### Routine for n = 1 ####################################
  #############################################################################
  Routine_for_n_1 <- function(N,sets,part){
  
    output <- sets[[N]]
    output <- matrix(c(output[1:N],output[length(output)]),1,(N+1))
    return(output)
  }
  #############################################################################
  #############################################################################
  
  #############################################################################
  ######################### Routine for n = 2 #################################
  #############################################################################
  Routine_for_n_2 <- function(N,sets,part){
    
    # Number of partitions
    n <- length(part)
    
    # Control procedure
    if (n!=2){
      stop("The number of partitions in wrong")
    }
    
    if (length(part)!=length(unique(part))){
      
      # Define the like set 
      alpha <- sets[[part[1]]]
      s1_alpha <- dim(alpha)[1]
      s2_alpha <- dim(alpha)[2]
      end_alpha <- s1_alpha + 1
      
      
      # Truncation point
      t <- s1_alpha/2
      
      # Define output
      first <- alpha[1:t,1:part[1]]
      first_like <- alpha[1:t,s2_alpha]
      second <- flip(alpha[(t+1):s1_alpha,1:part[1]])
      second_like <- rev(alpha[(t+1):s1_alpha,s2_alpha])
      output <- cbind(cbind(first,second),(first_like+second_like))
    }
    
    else{
      
      # Define the first like set 
      alpha <- sets[[part[1]]]
      s1_alpha <- dim(alpha)[1]
      s2_alpha <- dim(alpha)[2]
      end_alpha <- s1_alpha + 1
      
      # Define the second like set 
      beta <- sets[[part[2]]]
      s1_beta <- dim(beta)[1]
      s2_beta <- dim(beta)[2]
      end_beta <- s1_beta + 1
      
      # Define output
      first <- alpha[,1:part[1]]
      first_like <- alpha[,s2_alpha]
      second <- flip(beta[,1:part[2]])
      second_like <- rev(beta[,s2_beta])
      output <- cbind(cbind(first,second),(first_like+second_like))
      
    }
    
    #max <- matrix(output[which.max(output[,(N+1)]),],1,(N+1))
    index <- which(output[,(N+1)]==max(output[,(N+1)]))
    rows <- length(index)
    max <- output[index,]
    max <- matrix(max,rows,(N+1))
    
    return(max)
  }
  #############################################################################
  #############################################################################
  
  
  ###################################################################################
  ########################## Routine for identical partitions #######################
  ###################################################################################
  Routine_for_identical <- function(N,sets,part){
    
    # Set of value to be combined at the beginning
    start_set <- c(1:N)
    
    # Set of value to be combined in the various stages
    set <- start_set
    
    # Matrix for the excluded elements at the beginning
    excluded <- combValue(set,part[1])
    
    # Number of partitions
    n <- length(part)
    
    l <- dim(excluded)[1]
    
    tot_size <- N + 1
    
    ### Sub-function ###
    sub <- function(excluded,set,part,i,like_start,tot_size,n){
    
      # Combinations of the jth iteration
      set <- remove(set,excluded)
      comb <- combValue(set,part)
      s1 <- dim(comb)[1]
      t <- s1/length(c(i:n))
      comb <- comb[1:t,]
      excluded <- exclusion3(excluded,comb,t)
      
      beta <- sets[[part]]
      s1_beta <- dim(beta)[1]
      s2_beta <- dim(beta)[2]
      end_beta <- s1_beta + 1
      
      if (i < (n-1)){
        
        max_ll <- integer(0)
        
        for (jj in 1:t){
          zz <- 1
          while (zz <= s1_beta){
            diff <- sum(abs(comb[jj,] - beta[zz,1:part]))
            if (diff == 0){
              like <- like_start + beta[zz,s2_beta]
              zz <- end_beta
            }
            zz <- zz + 1
          }
          
          i <- i + 1
          
          h2 <- Recall(excluded[jj,],set,part,i,like,tot_size,n)
          max_ll <- rbind(max_ll,h2)
          
          i <- i - 1
        }
        
        index <- which(max_ll[,tot_size]==max(max_ll[,tot_size]))
        rows <- length(index)
        max <- max_ll[index,]
        max <- matrix(max,rows,tot_size)
        
      }
      else{
        
        max_ll <- integer(0)
        
        for (jj in 1:t){
          
          zz <- 1
          while (zz <= s1_beta){
            diff <- sum(abs(comb[jj,] - beta[zz,1:part]))
            if (diff == 0){
              like <- like_start + beta[zz,s2_beta]
              zz <- end_beta
            }
            zz <- zz + 1
          }
          
          wait <- remove(start_set,excluded[jj,])
          final <- c(excluded[jj,],wait)
          
          # Look for the like in the inner partitions
          zzz <- 1
          while (zzz <= s1_beta){
            diff <- sum(abs(wait - beta[zzz,1:part]))
            if(diff == 0){
              like <- like + beta[zzz,s2_beta]
              zzz <- end_beta
            }
            zzz <- zzz + 1
          }
          
          h3 <- matrix(c(final,like),1,tot_size)
          max_ll <- rbind(max_ll,h3)
        }
        
        index <- which(max_ll[,tot_size]==max(max_ll[,tot_size]))
        rows <- length(index)
        max <- max_ll[index,]
        max <- matrix(max,rows,tot_size)
        
        i <- i - 1
        return(max)
      }  
    }
    
        
    # Starting matrix
    l <- dim(excluded)[1]/n
    excluded <- excluded[1:l,]
    
    # Define the like set
    alpha <- sets[[part[1]]]
    s1_alpha <- dim(alpha)[1]
    s2_alpha <- dim(alpha)[2]
    end_alpha <- s1_alpha + 1
    
    max_l <- integer(0)
    
    for (j in 1:l){
      
      # look for the like of the first partition
      z <- 1
      while (z <= s1_alpha){
        diff <- sum(abs(excluded[j,] - alpha[z,1:part[1]]))
        if(diff==0){
          like_alpha <- alpha[z,s2_alpha]
          z <- end_alpha
        }
        z <- z + 1
      }
      i <- 2
      
      h1 <- sub(excluded[j,],set,part[1],i,like_alpha,tot_size,n)
      max_l <- rbind(max_l,h1)
      
    }
    
    index <- which(max_l[,tot_size]==max(max_l[,tot_size]))
    rows <- length(index)
    max <- max_l[index,]
    max <- matrix(max,rows,tot_size)
    
    return(max)
  }
  #############################################################################
  #############################################################################
  
  #############################################################################
  #################### Routine for different ##################################
  #############################################################################
  Routine_for_different <- function(N,sets,part){
  
    # Set of value to be combined at the beginning
    start_set <- c(1:N)
    
    # Set of value to be combined in the various stages
    set <- start_set
    
    # Matrix for the excluded elements at the beginning
    excluded <- combValue(set,part[1])
    
    # Number of partitions
    n <- length(part)
    
    l <- dim(excluded)[1]
    
    tot_size <- N + 1
    
    
    ### Sub-function ###
    sub <- function(excluded,set,part,i,like_start,tot_size,n){
      
      # Combinations of the jth iteration
      set <- remove(set,excluded)
      comb <- combValue(set,part[i])
      ll <- dim(comb)[1]
      excluded <- exclusion3(excluded,comb,ll)
      
      beta <- sets[[part[i]]]
      s1_beta <- dim(beta)[1]
      s2_beta <- dim(beta)[2]
      end_beta <- s1_beta + 1
      
      if (i < (n-1)){
        
        max_ll <- integer(0)
        
        for (jj in 1:ll){
          zz <- 1
          while (zz <= s1_beta){
            diff <- sum(abs(comb[jj,] - beta[zz,1:part[i]]))
            if (diff == 0){
              like <- like_start + beta[zz,s2_beta]
              zz <- end_beta
            }
            zz <- zz + 1
          }
          
          i <- i + 1
          
          h5 <- Recall(excluded[jj,],set,part,i,like,tot_size,n)
          max_ll <- rbind(max_ll,h5)
          
          i <- i - 1
        }
        
        index <- which(max_ll[,tot_size]==max(max_ll[,tot_size]))
        rows <- length(index)
        max <- max_ll[index,]
        max <- matrix(max,rows,tot_size)
        
      }
      else{
        
        theta <- sets[[part[n]]]
        s1_theta <- dim(theta)[1]
        s2_theta <- dim(theta)[2]
        end_theta <- s1_theta + 1
        
        max_ll <- integer(0)
        
        for (jj in 1:ll){
          zz <- 1
          while (zz <= s1_beta){
            diff <- sum(abs(comb[jj,] - beta[zz,1:part[i]]))
            if (diff == 0){
              like <- like_start + beta[zz,s2_beta]
              zz <- end_beta
            }
            zz <- zz + 1
          }
          
          wait <- remove(start_set,excluded[jj,])
          final <- c(excluded[jj,],wait)
          
          # Look for the like in the inner partitions
          zzz <- 1
          while (zzz <= s1_theta){
            diff <- sum(abs(wait - theta[zzz,1:part[n]]))
            if(diff == 0){
              like <- like + theta[zzz,s2_theta]
              zzz <- end_theta
            }
            zzz <- zzz + 1
          }
          
          h6 <- matrix(c(final,like),1,tot_size)
          max_ll <- rbind(max_ll,h6)
          
        }
        
        index <- which(max_ll[,tot_size]==max(max_ll[,tot_size]))
        rows <- length(index)
        max <- max_ll[index,]
        max <- matrix(max,rows,tot_size)
        
        return(max)
      }  
    }
    
    ### Routine for completely different partitions ###
    
    # Define the like set
    alpha <- sets[[part[1]]]
    s1_alpha <- dim(alpha)[1]
    s2_alpha <- dim(alpha)[2]
    end_alpha <- s1_alpha + 1
    
    # max_l <- matrix(0,l,tot_size)
    max_l <- integer(0)
    
    for (j in 1:l){
      
      # look for the like of the first partition
      z <- 1
      while (z <= s1_alpha){
        diff <- sum(abs(excluded[j,] - alpha[z,1:part[1]]))
        if(diff==0){
          like_alpha <- alpha[z,s2_alpha]
          z <- end_alpha
        }
        z <- z + 1
      }
      i <- 2
      
      h4 <- sub(excluded[j,],set,part,i,like_alpha,tot_size,n)
      max_l <- rbind(max_l,h4)
      
    }
    
    index <- which(max_l[,tot_size]==max(max_l[,tot_size]))
    rows <- length(index)
    max <- max_l[index,]
    max <- matrix(max,rows,tot_size)
    
    return(max)
  }
  #############################################################################
  #############################################################################
  
  #############################################################################
  ########################## Routine for the general case #####################
  #############################################################################
  Routine_general <- function(N,sets,part){
        
    # Set of value to be combined at the beginning
    start_set <- c(1:N)
    
    # Set of value to be combined in the various stages
    set <- start_set
    
    # Matrix for the excluded elements at the beginning
    excluded <- combValue(set,part[1])
    l <- dim(excluded)[1]
    
    # Number of partitions
    n <- length(part)
    
    # Parsing the partitions
    tab <- table(part)
    xu <- as.numeric(names(tab))
    xn <- as.vector(tab)
    xu <- rev(xu)
    xn <- rev(xn)
    sxn <- length(xn)
    tot_size <- N + 1
    
    ### Sub-routine ###
    sub <- function(excluded,set,i,k,like_start){
      
      if (i < sxn){
        
        if (k == 1){
          
          set <- remove(set,excluded)
          comb <- combValue(set,xu[i])
          ll <- dim(comb)[1]
          excluded <- exclusion3(excluded,comb,ll)
          
          beta <- sets[[xu[i]]]
          s1_beta <- dim(beta)[1]
          s2_beta <- dim(beta)[2]
          end_beta <- s1_beta + 1
          
          max_ll <- integer(0)
          
          for (jj in 1:ll){
            zz <- 1
            while (zz <= s1_beta){
              diff <- sum(abs(comb[jj,] - beta[zz,1:xu[i]]))
              if(diff == 0){
                like <- like_start + beta[zz,s2_beta]
                zz <- end_beta
              }
              zz <- zz + 1
            }
            i <- i + 1
            k <- xn[i]
            
            h8 <- Recall(excluded[jj,],set,i,k,like)
            max_ll <- rbind(max_ll,h8)  
            
            i <- i - 1
          }
          
          index <- which(max_ll[,tot_size]==max(max_ll[,tot_size]))
          rows <- length(index)
          max <- max_ll[index,]
          max <- matrix(max,rows,tot_size)
          return(max)
        }
        
        else{
          
          set <- remove(set,excluded)
          comb <- combValue(set,xu[i])
          t <- summorial(comb[,1])
          comb <- comb[1:t,]
          excluded <- exclusion3(excluded,comb,t)
          
          beta <- sets[[xu[i]]]
          s1_beta <- dim(beta)[1]
          s2_beta <- dim(beta)[2]
          end_beta <- s1_beta + 1
          
          max_ll <- integer(0)
          
          k <- k - 1
          
          for (jj in 1:t){
            
            zz <- 1
            while (zz <= s1_beta){
              diff <- sum(abs(comb[jj,] - beta[zz,1:xu[i]]))
              if(diff == 0){
                like <- like_start + beta[zz,s2_beta]
                zz <- end_beta
              }
              zz <- zz + 1
            }
            
            h9 <- Recall(excluded[jj,],set,i,k,like)
            max_ll <- rbind(max_ll,h9)
            
          }
          
          index <- which(max_ll[,tot_size]==max(max_ll[,tot_size]))
          rows <- length(index)
          max <- max_ll[index,]
          max <- matrix(max,rows,tot_size)
          
          k <- k + 1
          return(max)
        }
        
      }
      
      else{
        
        theta <- sets[[xu[i]]]
        s1_theta <- dim(theta)[1]
        s2_theta <- dim(theta)[2]
        end_theta <- s1_theta + 1
        
        if (k == 1){
          
          wait <- remove(start_set,excluded)
          final <- c(excluded,wait)
          
          zzz <- 1
          while (zzz <= s1_theta){
            diff <- sum(abs(wait - theta[zzz,1:xu[i]]))
            if(diff == 0){
              like <- like_start + theta[zzz,s2_theta]
              zzz <- end_theta
            }
            zzz <- zzz + 1
          }
          output <- matrix(c(final,like),1,tot_size)
          return(output)
        }
        
        else{
          set <- remove(set,excluded)
          comb <- combValue(set,xu[i])
          s1 <- dim(comb)[1]
          t <- s1/k
          comb <- comb[1:t,]
          excluded <- exclusion3(excluded,comb,t)
          
          max_ll <- integer(0)
          
          k <- k - 1
          
          for (jj in 1:t){
            
            zzz <- 1
            while (zzz <= s1_theta){
              diff <- sum(abs(comb[jj,] - theta[zzz,1:xu[i]]))
              if(diff == 0){
                like <- like_start + theta[zzz,s2_theta]
                zzz <- end_theta
              }
              zzz <- zzz + 1
            }
            
            h9 <- Recall(excluded[jj,],set,i,k,like)
            max_ll <- rbind(max_ll,h9)
            
          }
          
          index <- which(max_ll[,tot_size]==max(max_ll[,tot_size]))
          rows <- length(index)
          max <- max_ll[index,]
          max <- matrix(max,rows,tot_size)
          
          k <- k + 1
          return(max)
        }
        
      }
    }
    
    ### Routine for the general case ###
    
    #### First class partition - Single partition case ####
    if (xn[1] == 1){
      
      # Define the like set for the first partition
      alpha <- sets[[xu[1]]]
      s1_alpha <- dim(alpha)[1]
      s2_alpha <- dim(alpha)[2]
      end_alpha <- s1_alpha + 1
      
      max_l <- integer(0)
      
      for (j in 1:l){
        
        z <- 1
        while (z <= s1_alpha){
          diff <- sum(abs(excluded[j,] - alpha[z,1:xu[1]]))
          if(diff==0){
            like_alpha <- alpha[z,s2_alpha]
            z <- end_alpha
          }
          z <- z + 1
        }
        
        i <- 2
        k <- xn[i]
        
        h7 <- sub(excluded[j,],set,i,k,like_alpha)
        max_l <- rbind(max_l,h7)
        
      }
      
      index <- which(max_l[,tot_size]==max(max_l[,tot_size]))
      rows <- length(index)
      max <- max_l[index,]
      max <- matrix(max,rows,tot_size)
      
    }
    
    else {
      
      # Define the initial excluded vector
      l <- summorial(excluded[,1])
      excluded <- excluded[1:l,]
      
      # Define the like set
      alpha <- sets[[part[1]]]
      s1_alpha <- dim(alpha)[1]
      s2_alpha <- dim(alpha)[2]
      end_alpha <- s1_alpha + 1
      
      # max_l <- matrix(0,l,tot_size)
      max_l <- integer(0)
      
      i <- 1
      k <- xn[1] - 1
      for (j in 1:l){
        
        # look for the like of the first partition
        z <- 1
        while (z <= s1_alpha){
          diff <- sum(abs(excluded[j,] - alpha[z,1:xu[i]]))
          if(diff==0){
            like_alpha <- alpha[z,s2_alpha]
            z <- end_alpha
          }
          z <- z + 1
        }
        
        h10 <- sub(excluded[j,],set,i,k,like_alpha)
        max_l <- rbind(max_l,h10)
        
      }
      
      index <- which(max_l[,tot_size]==max(max_l[,tot_size]))
      rows <- length(index)
      max <- max_l[index,]
      max <- matrix(max,rows,tot_size)
      
    }
    
    return(max)
  }
  #############################################################################
  #############################################################################
  
  #############################################################################
  ########################## Routine output ###################################
  #############################################################################
  Routine_output <- function(dim,output,sets,part){
    
    s <- dim + 1
    
    # Number of optimal solutions
    index <- which(output[,s]==max(output[,s]))
    nr <- length(index)
    
    # Optimal solutions
    max <- matrix(output[index,1:s],1,s)
    i <- output[index,(s+1)]
    
    # Prints the number of optimal solutions
    cat("The number of optimal solutions is", nr, "\n")
    
    results <- list()
    for (j in 1:nr){
      
      # Prints the results of j
      cat("These are the clusters for solution", j, "\n")
      pt <- part[[i[j]]]
      s1 <- length(pt)
      u <- 0
      intermediate <- list()
      like <- matrix(0,s1,2)
      for (k in 1:s1){
        alpha <- sets[[pt[k]]]
        s1_alpha <- dim(alpha)[1]
        s2_alpha <- dim(alpha)[2]
        d <- u + 1
        u <- u + pt[k] 
        zz <- 1
        end_alpha <- s1_alpha + 1
        while (zz <= s1_alpha){
          diff <- sum(abs(max[j,d:u] - alpha[zz,1:pt[k]]))
          if (diff == 0){
            like[k,1] <- k
            like[k,2] <- alpha[zz,s2_alpha]
            intermediate[[k]] <- alpha[zz,1:pt[k]]
            zz <- end_alpha
          }
          zz <- zz + 1
          
        }
      }
      
      results <- list()
      s_int <- length(intermediate)-1
      for (kk in 1:s_int){
        index <- which.max(like[,2])
        cat("Cluster",kk,"\n")
        print(intermediate[[like[index,1]]])
        results[[kk]] <- intermediate[[like[index,1]]]
        like <- like[-index,]
      }
      cat("Cluster",s1,"\n")
      print(intermediate[[like[1]]])
      results[[(s_int+1)]] <- intermediate[[like[1]]]
      cat("The global likelihood is",max[j,s])
      results[(s_int+2)] <- max[j,s]
    }
    return(results)
  }
  #############################################################################
  #############################################################################
  
  #############################################################################
  ########################## MAIN ALGORITHM ###################################
  #############################################################################
  time <- system.time({
    
    # Define the copula sets
    sets <- Routine_Copula(data,dim,copula)
    #load("C:/Users/Luca/Documents/R/trade12M.RData")
      
    # Define the partitions
    partitions <- clean(dim)
    n <- length(partitions)
    
    # Define the matrix with the maximum for each partition
    s <- dim + 1
    
    # output <- matrix(0,n,s)
    output <- integer(0)
    
    # Find the maximun for each partition
    for (i in 1:n){
      
      part <- partitions[[i]]
      sn <- length(part)
      
      if (sn == 1){
        
        routine <- Routine_for_n_1(dim,sets,part)
        output <- rbind(output,cbind(routine,i))
        
      }else if (sn == 2){
        
        routine <- Routine_for_n_2(dim,sets,part)
        output <- rbind(output,cbind(routine,i))
        
      }else if (sn > 2 & length(unique(part)) == 1){
        
        routine <- Routine_for_identical(dim,sets,part)
        output <- rbind(output,cbind(routine,i))
        
      }else if (sn > 2 & length(unique(part)) == sn){
        
        routine <- Routine_for_different(dim,sets,part)
        output <- rbind(output,cbind(routine,i))
          
      }else{
        
        routine <- Routine_general(dim,sets,part)
        output <- rbind(output,cbind(routine,i))
        
      }
    }    
      
    results <- Routine_output(dim,output,sets,partitions)
  
  })
  
  return(results)
}
###############################################################################
###############################################################################